/* -*- linux-c -*- */


/*
    SMBus.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1998  Alexander Larsson <alla@lysator.liu.se>,
    Frodo Looijaard <frodol@dds.nl> and Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *  SMBus driver for LINUX
 *
 * Copyright (c) 1998 Philip Edelbrock
 * 
 * part of a module which provides a 
 * /proc/sensors file which shows
 * some information about your system.
 * Voltages, temperature and fan speeds
 * are currently reported.
 *
 *
 * please report any bugs/patches to me:
 *  Philip Edelbrock <phil@netroedge.com>
 *
 *
 * Reference:
 *   National Semiconductor
 *   LM75 Digital Temperature Sensor and Thermal Watchdog with
 *   Two-Wire Interface, PDF, April 1997.
 *   http://www.national.com/ds/LM/LM75.pdf
 *
 *   Intel
 *   82371AB PCI-TO-ISA / IDE XCELERATOR (PIIX4), PDF, April 1997.
 *   ftp://ftp.intel.com
 *   
 *   System Management Bus Specification, Rev. 1.0, 1996.
 *   Contributors:
 *   Benchmarq Microelectronics Inc ., Duracell Inc.,
 *   Energizer Power Systems, Intel Corporation, Linear Technology Corporation,
 *   Maxim Integrated Products, Mitsubishi Electric Corporation,
 *   National Semiconductor Corporation, Toshiba Battery Co.,
 *   Varta Batterie AG, All rights reserved.
 *   http://www.sbs-forum.org/
 */
#include <linux/config.h>

#ifndef SMBUS_MODULE
/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__
#include <linux/version.h>
#endif

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <asm/semaphore.h>

#include "smbus.h"
#include "lmsmbus.h"
#include "compat.h"

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0))
#include <linux/bios32.h>
#endif

/*
	Global variables
*/

static unsigned short PIIX4_smba = 0;
static struct semaphore smbus_sem = MUTEX;

int SMBus_Initialized=0;

/*
	External function which are not in smbus.h
*/

#ifdef SMBUS_MODULE
extern int init_module(void);
extern void cleanup_module(void);
#endif

/*
	Local functions
*/

static void do_pause( unsigned amount );

#ifdef SMBUS_MODULE

static int forced=0;

MODULE_AUTHOR("Alexander Larsson <alla@lysator.liu.se>, Frodo Looijaard <frodol@dds.nl>, Philip Edelbrock <phil@Ren.netroedge.com>");

MODULE_DESCRIPTION("Driver for SMBus (System Management Bus) access");

MODULE_PARM(forced,"i");
MODULE_PARM_DESC(forced,"If set, it forces the SMBus module to load, even if no SMBus is detected");

int init_module(void)
{
  int res=SMBus_Init();
  if (res)
    printk("SMBus not detected\n");
  else
    printk("SMBus detected and initialized\n");
  if (! forced)
    return res;
  else
    return 0;
}

void cleanup_module(void)
{
  SMBus_Cleanup();
}
#endif

int SMBus_Init(void)
{
  int error_return=0;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
  struct pci_dev *PIIX4_dev;
#else
  unsigned char PIIX4_bus, PIIX4_devfn;
#endif

#ifdef DEBUG
  unsigned char temp;
#endif

  if (pci_present() == 0) {
    printk("SMBus: Error: No PCI-bus found!\n");
    error_return=-ENODEV;
  } else {

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
    PIIX4_dev = pci_find_device(PCI_VENDOR_ID_INTEL, 
                                 PCI_DEVICE_ID_INTEL_82371AB_0, NULL);
    if(PIIX4_dev == NULL) {
#else
    if(pcibios_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371AB_0, 
                           0, &PIIX4_bus, &PIIX4_devfn)) {
#endif
      printk("SMBus: Error: Can't detect PIIX4!\n");
      error_return=-ENODEV;
    } else {
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
      /* For some reason, pci_read_config fails here sometimes! */
      pcibios_read_config_word(PIIX4_dev->bus->number, PIIX4_dev->devfn | 3, 
                               SMBBA, &PIIX4_smba);
#else
      pci_read_config_word_united(PIIX4_dev, PIIX4_bus ,PIIX4_devfn | 3,
                                  SMBBA,&PIIX4_smba);
#endif
      PIIX4_smba &= 0xfff0;
      if (check_region(PIIX4_smba, 8)) {
        printk("SMBus: Error: PIIX4_smb region 0x%x already in use!\n",
               PIIX4_smba);
        error_return=-ENODEV;
      } else {
        /* Everything is happy, let's grab the memory and set things up. */
        request_region(PIIX4_smba, 8, "SMBus");       
        SMBus_Initialized=1;
#ifdef DEBUG
        pci_read_config_byte_united(PIIX4_dev, PIIX4_bus, PIIX4_devfn | 3, 
                                    SMBHSTCFG, &temp);
        if ((temp & 1) == 0)
          printk("SMBUS: Error: Host SMBus controller not enabled!\n");
        if ((temp & 0x0E) == 8)
          printk("SMBUS: PIIX4 using Interrupt 9 for SMBus.\n");
        else if (((temp & 0x0E) == 0)
          printk("SMBUS: PIIX4 using Interrupt SMI# for SMBus.\n");
        else 
          printk( "SMBUS: PIIX4: Illegal Interrupt configuration (or code out of date)!\n");
        pcibios_read_config_byte_united(PIIX4_dev, PIIX4_bus, PIIX4_devfn | 3, 
                                        SMBREV, &temp);
        printk("SMBUS: SMBREV = 0x%X\n",temp);
#endif
      }
    }
  }
  if (error_return)
    printk("SMBus setup failed.\n");
#ifdef DEBUG
  else
    printk("PIIX4_smba = 0x%X\n",PIIX4_smba);
#endif
  return error_return;
}

/*
 *      Clean-up the SMBus code.
 */
void SMBus_Cleanup(void){
	if (SMBus_Initialized) {
        	release_region(PIIX4_smba, 8); 
		SMBus_Initialized=0;
	}
}

/*
 *      Software pause .
 */
void do_pause( unsigned amount )
{
      current->state = TASK_INTERRUPTIBLE;
      current->timeout = jiffies + amount;
      schedule();
}

/*
 *      Write byte value to SMBus device at given address.
 */
int SMBus_Write(int addr, long value, int size) {

int HOST_BUSY;
int timeout=0;
int result=0;

    if (! SMBus_Initialized) {
       printk("SMBus: Attempt made to use uninitialized SMBus routines!\n");
       return -1;
    }
  /* Grab the mutex */
  do_pause(1);
  down(&smbus_sem);
/* Setup protocol type, address, device command, data, and control bits */
    outb_p( value & 0x0FF, SMBHSTCMD);
    outb_p( (value & 0x0FF00) >> 8, SMBHSTDAT0);
    outb_p( (value & 0x0FF0000) >> 16, SMBHSTDAT1);
    outb_p( (size & 0x1C) + (ENABLE_INT9 & 1), SMBHSTCNT); /* SMBus operation size and Int config */
    if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
	printk("SMBus: SMBus_Write_byte: SMBus busy. Resetting... ");
#endif
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
	if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
		printk("Failed!\n");
#endif
		return -1;
	} else {
#ifdef DEBUG
		printk("Successfull!\n");
#endif
       }
    }
    outb_p( (addr << 1) & 0x0FE, SMBHSTADD);
/* Set the start bit */
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); 
/* start the transaction by setting bit 6 */
/* Wait for a fraction of a second! */
   do_pause(1);
/* Poll Host_busy bit */
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   while (HOST_BUSY & (timeout++ < MAX_TIMEOUT)) {
/* Wait for a while and try again*/
        do_pause(1);
	HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   }
#ifdef DEBUG
   if (timeout++ >= MAX_TIMEOUT)
	   printk("SMBus: SMBus_Write_byte: SMBus Timeout!\n");
/*   printk("SMBus: SMBus_Write_byte: time cycles for transaction: %d (0x%X)\n",timeout,addr); */
#endif
/* Sense for no device at the address */
   if (inb_p(SMBHSTSTS) == 0x04)  {
        result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Write_byte: SMBus error: no response from 0x%X !\n",addr);
#endif
   }
  if (inb_p(SMBHSTSTS) & 0x010) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Write: SMBus error: Failed bus transaction (0x%X).\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) & 0x08) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Write: SMBus error: Bus collision (0x%X).\n",addr);
#endif
	printk("SMBus: SMBus_Write: Bus collision! SMBus may be locked until next hard reset. (sorry!)\n");
   }
   if (inb_p(SMBHSTSTS) & 0x04) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Write: SMBus error: Device Error (0x%X).\n",addr);
#endif
   }
    if (inb_p(SMBHSTSTS) != 0x00)
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
  /* Release the mutex */
   up(&smbus_sem);
   return result;
}


/*
 *      Read value from SMBus device at given address.
 */
int SMBus_Read(int addr, int command, int size) {

int HOST_BUSY;
int timeout=0;
int result=0;

    if (! SMBus_Initialized) {
       printk("SMBus: Attempt made to use uninitalized SMBus routines!\n");
       return -1;
    }
  /* Grab the mutex */
  do_pause(1);
  down(&smbus_sem);
/* Setup protocol type, address, device command, data, and control bits */
    outb_p( command & 0x0FF, SMBHSTCMD);
    outb_p( 0x00, SMBHSTDAT0);
    outb_p( 0x00, SMBHSTDAT1);
    outb_p( (size & 0x1C) + (ENABLE_INT9 & 0x01), SMBHSTCNT); /* SMBus operation size and interrupt.
    							Everything else should be zero for now */
    if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus busy. Resetting... ");
#endif
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
	if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
		printk("Failed!\n");
#endif
		return -1;
	} else {
#ifdef DEBUG
		printk("Successfull!\n");
#endif
       }
    }
    outb_p( (addr << 1) | 0x01, SMBHSTADD);
/* Set the start bit */
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); 
/* start the transaction by setting bit 6 */
/* Wait for a fraction of a second! */
   do_pause(1);
/* Poll Host_busy bit */
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   while (HOST_BUSY & (timeout++ < MAX_TIMEOUT)) {
/* Wait for a while and try again*/
        do_pause(1);
	HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   }
#ifdef DEBUG
   if (timeout++ >= MAX_TIMEOUT)
	   printk("SMBus: SMBus_Read: SMBus Timeout!\n");
/*   printk("SMBus: SMBus_Read: time cycles for transaction: %d (0x%X)\n",timeout,addr); */
#endif
   if (! (inb(SMBHSTSTS) & 0x01)) {
        result=(inb(SMBHSTDAT0) << 8) | inb(SMBHSTDAT1);
   }
/* Sense for no device at the address */
   if (inb_p(SMBHSTSTS) == 0x04)  {
        result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: no response from 0x%X !\n",addr);
#endif
   }

   if (inb_p(SMBHSTSTS) & 0x010) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Failed bus transaction (0x%X).\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) & 0x08) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Bus collision (0x%X).\n",addr);
#endif
	printk("SMBus: SMBus_Read: Bus collision! SMBus may be locked until next hard reset. (sorry!)\n");
   }
   if (inb_p(SMBHSTSTS) & 0x04) {
   	result=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Device Error (0x%X).\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) != 0x00)
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
  /* Release the mutex */
   up(&smbus_sem);
   return result;
}

/*
 *      Read value from SMBus device at given address.
 */
void SMBus_Read_Block(int addr, int command, char *result) {

int HOST_BUSY;
int timeout=0;
int i,count;

    if (! SMBus_Initialized) {
       printk("SMBus: Attempt made to use uninitalized SMBus routines!\n");
       result[0]=-1;
    }
/* Initialize the array with zero */
  for (i=0;i<34;i++)
  	result[i]=0;
  do_pause(1);
  /* Grab the mutex */
  down(&smbus_sem);
/* Setup protocol type, address, device command, data, and control bits */
    outb_p( command & 0x0FF, SMBHSTCMD);
    outb_p( 0x00, SMBHSTDAT0);
    outb_p( 0x00, SMBHSTDAT1);
    outb_p( (SMBUS_BLOCK_DATA & 0x1C) + (ENABLE_INT9 & 0x01), SMBHSTCNT); /* SMBus operation size and interrupt.
    							Everything else should be zero for now */
    if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
	printk("SMBus: SMBus_Read_Block: SMBus busy. Resetting... ");
#endif
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
	if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
		printk("Failed!\n");
#endif
		result[0]=-1;
		return;
	} else {
#ifdef DEBUG
		printk("Successfull!\n");
#endif
       }
    }
    outb_p( (addr << 1) | 0x01, SMBHSTADD);
/* Set the start bit */
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); 
/* start the transaction by setting bit 6 */
/* Wait for a fraction of a second! */
   do_pause(1);
/* Poll Host_busy bit */
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   while (HOST_BUSY & (timeout++ < MAX_TIMEOUT)) {
/* Wait for a while and try again*/
        do_pause(1);
	HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   }
#ifdef DEBUG
   if (timeout++ >= MAX_TIMEOUT)
	   printk("SMBus: SMBus_Read: SMBus Timeout!\n");
/*   printk("SMBus: SMBus_Read: time cycles for transaction: %d (0x%X)\n",timeout,addr); */
#endif
   if (! (inb(SMBHSTSTS) & 0x01)) {
   	count=inb(SMBHSTDAT0);
	result[0]=count;
   	for (i=1; i<=count; i++)
   		result[i]=inb(SMBBLKDAT);
   }
/* Sense for no device at the address */
   if (inb_p(SMBHSTSTS) == 0x04)  {
        result[0]=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: no response from 0x%X !\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) & 0x010) {
   	result[0]=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Failed bus transaction (0x%X).\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) & 0x08) {
   	result[0]=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Bus collision (0x%X).\n",addr);
#endif
	printk("SMBus: SMBus_Read: Bus collision! SMBus may be locked until next hard reset. (sorry!)\n");
   }
   if (inb_p(SMBHSTSTS) & 0x04) {
   	result[0]=-1;
#ifdef DEBUG
	printk("SMBus: SMBus_Read: SMBus error: Device Error (0x%X).\n",addr);
#endif
   }
   if (inb_p(SMBHSTSTS) != 0x00)
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
  /* Release the mutex */
   up(&smbus_sem);
}
